/*
    This source code contains routines to get, calculate and
    rewrite GameBoy CheckSums.  It is meant that this source
    to be used as a learning tool and not a tool of piracy.

    *BorlandC 3.1 was used to compile this successfully.

    It is a crime to redistribute these routines in a commercial
    venture of any kind without permission or licensing agreement.
    Contact us at our web page for more information on licensing.

    (C)opyright 1995,96 Damaged Cybernetics

    If you have any questions or comments, please contact the
     following ppl via the Internet:

	Donald Moore   (MindRape) mindrape@goodnet.com
	Donald Staheli (Royce)     staheli@goodnet.com

    Web Page: www.futureone.com/~damaged

 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <conio.h>
#include "gameboy.h"

/* Prototypes */
void CorrectGBCheckSum(char *filename);

/* misc character strings for title and help */
char szTitle[] =
  {
    "GBSUM Version 1.2\n"
    "(C) 1995,96 Damaged Cybernetics (www.goodnet.com/~staheli)\n"
  };

char szHelp[] =
  {
    "Usage: GBSUM <game.gb>\n\n"
    "GBSUM will verify that your gameboy rom images contain the correct\n"
    "checksum value.  It currently does not support complement checks.\n"
  };



/*
   void CorrectGBCheckSum(char *filename)
   ENT: char *filename - GameBoy file to verify checksum
   RET: <nothing>

   Grab the checksum value and compare it against our calculated value.
   If the value is wrong, ask the user if he wants to fix it.

 */
void CorrectGBCheckSum(char *filename)
{ FILE *source;
  unsigned int uiStoredCheckSum;
  unsigned int uiCalcCheckSum;
  unsigned int uiOffset;

  if ((source = fopen(filename,"r+b"))==NULL)
    {
      printf("Cannot open input file: %s\n",filename);
      exit(2);
    }

  switch(DetectGameBoy(source))
    {
      case DETECT_GAMEBOY_INVALID:puts("Detected: Invalid GameBoy image.");
                                  exit(0);  /* bail */

      case DETECT_GAMEBOY_HEADER :puts("Detected: GameBoy image with header.");
                                  uiOffset = HEADER_SIZE;  /* adjust for difference in offsets */
                                  break;

      case DETECT_GAMEBOY_RAW    :puts("Detected: Raw GameBoy image.");
                                  uiOffset = 0;            /* no header, no adjustment         */
                                  break;
    };

  uiStoredCheckSum = GetGBCheckSum(source,uiOffset);      /* get stored checksum */
  uiCalcCheckSum = CalculateGBCheckSum(source,uiOffset);  /* Calculate it        */

  printf("GB Stored CheckSum    : 0x%04X\n",uiStoredCheckSum);
  printf("GB Calculated CheckSum: 0x%04X\n",uiCalcCheckSum);


  if (uiStoredCheckSum != uiCalcCheckSum)
    {
       /* since the checksums don't match, ask the user if he wants to
          repair it by writing what we calculated with the CalculateGBCheckSum
          routine.
        */
       printf("\nCheckSum is invalid!  Do you want to repair it? (Y/N) \n");

       if (toupper(getch()) == 'Y')
         {
           PutGBCheckSum(source,uiCalcCheckSum);
           puts("Fixed");
         }
       else
         puts("Not Fixed");
    }
  else
    printf("\nCheckSum is valid!\n");

  fclose(source);      /* close up our file handles */
};


void main(int argc,char *argv[])
{
  puts(szTitle);              /* print out our little title   */

  if (argc != 2)              /* must have 2 arguments!       */
    {
      puts(szHelp);           /* well print out the help file */
      exit(1);                /* exit the program             */
    }

  CorrectGBCheckSum(argv[1]);  /* Go verify it!                */
}
